#!/usr/bin/env python3

import re
import urllib.request
import asyncio
import aiohttp
import time


async def get_content(name):
    """
    Функция возвращает содержимое вики-страницы name из русской Википедии.
    В случае ошибки загрузки или отсутствия страницы возвращается None.
    """
    link = u"https://ru.wikipedia.org/wiki/" + urllib.request.quote(name)
    try:
        async with aiohttp.ClientSession() as session:
            async with session.get(link) as response:
                return await response.text()
    except urllib.request.HTTPError:
        return None


def extract_content(page):
    """
    Функция принимает на вход содержимое страницы и возвращает 2-элементный
    tuple, первый элемент которого — номер позиции, с которой начинается
    содержимое статьи, второй элемент — номер позиции, на котором заканчивается
    содержимое статьи.
    Если содержимое отсутствует, возвращается (0, 0).
    """
    if page is None:
        return 0, 0
    res = re.search(
        '<div id="bodyContent"(.)*<div class="visualClear">',
        page,
        re.DOTALL)
    if res is None:
        return 0, 0
    return res.start(0), res.end(0)


def extract_links(page, begin, end):
    """
    Функция принимает на вход содержимое страницы и начало и конец интервала,
    задающего позицию содержимого статьи на странице и возвращает все имеющиеся
    ссылки на другие вики-страницы без повторений и с учётом регистра.
    """
    if page is None or (begin == 0 and end == 0):
        return None
    text = page[begin:end]
    start = re.search('(.)*?<p><b>(.)*?</b>', text, re.DOTALL)
    if start is not None:
        text = text[start.start(1):]
    text = re.sub(r'\(.*?\)', " ", text)
    text = re.sub(r'<i>.+?</i>', " ", text)
    links = re.findall(
        r'<(?:a|A)\s+?(?:h|H)ref=(?:"|\')/wiki/([_%\w\d-]*?)(?:"|\')',
        text)
    res = []
    for link in links:
        unquoted_link = urllib.request.unquote(link)
        if res.count(unquoted_link) == 0:
            res.append(unquoted_link)
    return res


def find_chain(page):
    """
    Функция принимает на вход название начальной страницы
    возвращает содержимое всех ссылок на ней.
    """
    loop = asyncio.get_event_loop()

    page = loop.run_until_complete(get_content(page))

    content = extract_content(page)
    links = extract_links(page, content[0], content[1])

    coroutines = [get_content(link) for link in links]
    wait_task = asyncio.wait(coroutines)
    result, _ = loop.run_until_complete(wait_task)

    loop.close()
    return [f.result() for f in result]


def main():
    start = time.time()
    result = find_chain('Наука')
    finish = time.time() - start
    print(finish)
    with open('output.txt', 'w', encoding='utf-8') as f:
        f.write("\n\n\n".join(result))


if __name__ == '__main__':
    main()
